# -*- coding: utf-8 -*-

from threading import Thread

from adaura_serial import Serial
from adaura_serial.tools.list_ports import comports
import wx


# Possible settings for a new line character
NEWLINE_CR = 0
NEWLINE_LF = 1
NEWLINE_CRLF = 2


def CreateSerial():
    """
    Serial initialization
    """
    s = Serial()

    s.baudrate = 9600
    s.timeout = 0.5
    #s.dtr = True
    #s.rts = True

    return s

def GetPortsList():
    return [p.device for p in comports()]


def WriteLine(serial, data, newline=NEWLINE_CRLF):
    if len(data) < 1:
        return

    serial.write(data.encode('utf-8', 'replace'))

    # Send appropriate newline character
    if newline == NEWLINE_CR:
        serial.write(b'\r')
    elif newline == NEWLINE_LF:
        serial.write(b'\n')
    elif newline == NEWLINE_CRLF:
        serial.write(b'\r\n')

# Custom wxWidgets events
RECEIVED = wx.NewEventType()
READERROR = wx.NewEventType()

EVT_RECEIVED = wx.PyEventBinder(RECEIVED, 0)
EVT_READERROR = wx.PyEventBinder(READERROR, 0)


class BaseReaderEvent(wx.PyCommandEvent):
    """
    Superclass for our custom events
    """
    def __init__(self, windowID, data):
        super(BaseReaderEvent, self).__init__(self.eventType, windowID)
        self.data = data
    
    def Clone(self):
        self.__class__(self.GetId(), self.data)


class BytesReceivedEvent(BaseReaderEvent):
    eventType = RECEIVED

    def __init__(self, windowID, data):
        super(BytesReceivedEvent, self).__init__(self.eventType, windowID)
        self.data = data.decode('utf-8', 'replace')


class ReadErrorEvent(BaseReaderEvent):
    eventType = READERROR


class InputThread(Thread):
    """
    Background reciever thread
    Works asynchronously from the main window
    """
    def __init__(self, parent, serial, newline=NEWLINE_CRLF):
        super(InputThread, self).__init__()

        self.newline = newline
        self.serial = serial
        self.parent = parent
        self._stop = False

    def _transform(self, data):
        """
        Sanitize received data
        """
        if self.newline == NEWLINE_CR:
            data = data.replace(b'\r', b'\n')
        elif self.newline == NEWLINE_LF:
            pass
        elif self.newline == NEWLINE_CRLF:
            data = data.replace(b'\r\n', b'\n')
        
        return data

    def run(self):
        while not self._stop:
            try:
                data = self.serial.read(self.serial.in_waiting or 1)

                event = BytesReceivedEvent(
                    windowID=self.parent.GetId(),
                    data=self._transform(data)) if data else None
            except Exception as e:
                self.abort()
                event = ReadErrorEvent(
                    windowID=self.parent.GetId(),
                    data=str(e))
            finally:
                if event:
                    self.parent.GetEventHandler().AddPendingEvent(event)

    def abort(self):
        self._stop = True
